from key_spyder.defaults import NOW

from os import path, makedirs
from urllib.parse import urlparse
from urllib import robotparser, request

from bs4 import BeautifulSoup
import pandas as pd


class Sitemapper:
    def __init__(self, url):
        self.url = urlparse(url)
        self.__filename = f"{self.url.netloc}_sitemap_{NOW}.csv"
        self.__sitemap_index = self.__get_sitemap_url()
        self.all_urls = self.__get_all_urls(self.__sitemap_index)

    def __get_sitemap_url(self):
        robots = f"{self.url.scheme}://{self.url.netloc}/robots.txt"
        rp = robotparser.RobotFileParser()
        rp.set_url(robots)
        rp.read()
        return rp.site_maps()[0]

    @staticmethod
    def __get_sitemap(url):
        r = request.urlopen(url)
        xml = BeautifulSoup(r, 'lxml-xml', from_encoding=r.info().get_param('charset'))
        return xml

    @staticmethod
    def __get_sitemap_type(xml):
        sitemapindex = xml.find('sitemapindex')
        sitemap = xml.find('sitemap')
        if sitemapindex:
            return 'sitemapindex'
        elif sitemap:
            return 'sitemap'
        else:
            return None

    @staticmethod
    def __sitemap_to_df(xml, name=None):
        df = pd.DataFrame(
            columns=['loc', 'changefreq', 'priority', 'domain', 'sitemap_name'])
        urls = xml.find_all("url")
        for url in urls:
            if xml.find("loc"):
                loc = url.findNext("loc").text
                parsed_uri = urlparse(loc)
                domain = f'{parsed_uri.netloc}'
            else:
                loc = ''
                domain = ''

            if xml.find("changefreq"):
                changefreq = url.findNext("changefreq").text
            else:
                changefreq = ''

            if xml.find("priority"):
                priority = url.findNext("priority").text
            else:
                priority = ''

            if name:
                sitemap_name = name
            else:
                sitemap_name = ''

            row = {
                'loc':          loc,
                'changefreq':   changefreq,
                'priority':     priority,
                'domain':       domain,
                'sitemap_name': sitemap_name,
            }

            df.loc[len(df)] = list(row.values())
        return df

    @staticmethod
    def __get_child_sitemaps(xml):
        sitemaps = xml.find_all('sitemap')
        return [sitemap.findNext('loc').text for sitemap in sitemaps]

    def __get_all_urls(self, url):
        xml = self.__get_sitemap(url)
        sitemap_type = self.__get_sitemap_type(xml)

        if sitemap_type == 'sitemapindex':
            sitemaps = self.__get_child_sitemaps(xml)
        else:
            sitemaps = [url]

        df = pd.DataFrame(columns=['loc', 'changefreq', 'priority', 'domain', 'sitemap_name'])
        for sitemap in sitemaps:
            sitemap_xml = self.__get_sitemap(sitemap)
            df_sitemap = self.__sitemap_to_df(sitemap_xml, name=sitemap)
            df = pd.concat([df, df_sitemap], ignore_index=True)

        return df

    def to_csv(self, filepath):
        sitemap_path = path.join(filepath, "sitemaps")
        if not path.exists(sitemap_path):
            makedirs(sitemap_path)
        self.all_urls.to_csv(f"{sitemap_path}/{self.__filename}", index=False)
